var gc = gc || {};
gc.services = gc.services || {};
// [ Start of file stream code
var mystream;
var mywriter;
var myTextEnc = new TextEncoder();
var saveStart = function (filename) {
    if (!filename) filename = 'mydata.csv';
    mystream = streamSaver.createWriteStream(filename);
    saveAbort();
    mywriter = mystream.getWriter();
    saveData(myTextEnc.encode('Temperature, Time\n'));
};
var saveStop = function () {
    if (mywriter) {
        mywriter.close();
        mywriter = null;
    }
};
var saveAbort = function () {
    if (mywriter) {
        mywriter.abort('reason');
        mywriter = null;
    }
};
var saveData = function (data) {
    if (mywriter) {
        mywriter.write(data);
    }
};
window.addEventListener('unload', (event) => {
    //  console.log('unload', event);
    saveAbort(); // seems ok but please test.
});
// end of file stream code


const readTemperature = () => {
    // read temperature from device
    const u2a = document.querySelector('ti-model-register').getCodec();
    if (u2a) {
        u2a.sendCommandPacket(0x19, [1, 1, 1, 1, 2]);
        return u2a.readResponse(u2a.sendCommandPacket(0x1A, [12, 0])).then(packet => {
            var payload = packet.slice(8);
            
            // decode ADC
            var Vref = 3.3;
            var hibyte = payload[0];
            var lobyte = payload[1];
            var word = (hibyte << 8) | (lobyte & 0xFF);
            var voltage = (word * Vref) / (Math.pow(2, 12));
            
            // convert voltage to temperature
            var temperatureCelsius = -1481.96 + Math.sqrt(2.1962 * Math.pow(10, 6) + ((1.8639 - voltage) / (3.88 * Math.pow(10, -6))));
            var temperatureFahrenheit = (temperatureCelsius * 1.8) + 32;

            // console.log(payload, hibyte, lobyte, word);
            
            // get date for file stream
             var d = new Date();
             timestamp = d.getFullYear() + '/' + d.getMonth() + '/' + d.getDate() + ' ' + d.getHours() + ':' + d.getMinutes() + ':' + d.getSeconds();
             
             saveData(myTextEnc.encode(temperatureCelsius + ',' + timestamp + '\n'));
            

            //console.log(temperatureCelsius, temperatureFahrenheit);
            return temperatureCelsius.toFixed(3); // round temp to 3 decimal places
        });
    } else {
        return Q(0);
    }
};


/* setup widget event listener */
var initComplete = false;
var templateObj;
var refreshGraphRate = 0;
var hInterval = null;

const poll = () => {
    if (hInterval) {
        clearInterval(hInterval);
        hInterval = null;
    }

    if (refreshGraphRate >= 0) {
        hInterval = setInterval(() => {
            readTemperature().then(temperature => {
                const tempBinding = gc.databind.registry.getBinding('prop.value');
                tempBinding.setValue(+temperature);
            });
        }, refreshGraphRate);
    }
    templateObj.$.ti_widget_scalargraph.tickOn = refreshGraphRate >= 0 ? true : false;
    templateObj.$.ti_widget_scalargraph.stop();
    
    if (templateObj.$.ti_widget_scalargraph.tickOn)
        templateObj.$.ti_widget_scalargraph.start();
};


var init = function () {
    templateObj = document.querySelector('#template_obj');
    templateObj.addEventListener('dom-change', () => {
        if (initComplete) return;

        templateObj.async(() => {
            initComplete = true;

            refreshGraphRate = templateObj.$.ti_widget_droplist.selectedValue;
            poll();
            
            // droplist refresh stuff
            templateObj.$.ti_widget_droplist.addEventListener("selected-value-changed", ({ detail }) => {
                refreshGraphRate = detail.value;
                poll();
            });
            
            // save start and save stop clicks
            //templateObj.$.ti_widget_button_save_start.addEventListener("click", saveStart);
            
            //templateObj.$.ti_widget_button_save_stop.addEventListener("click", saveStop); 

            // menu button
            templateObj.toggleMenu = function (event) {
                templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
            };
            templateObj.$.ti_widget_icon_button_menu.addEventListener('click', templateObj.toggleMenu);
            templateObj.$.ti_widget_label_menu.addEventListener('click', templateObj.toggleMenu);


            // "learn more" button function
            if (templateObj.$.ti_widget_label_learnmore) {
                templateObj.$.ti_widget_label_learnmore.addEventListener("click", function (event) {
                    templateObj.$.ti_widget_label_productname.fontColor = "#444";
                    templateObj.$.ti_widget_label_productdesc.fontColor = "#444";
                    templateObj.$.ti_widget_label_learnmore.setAttribute('hidden', true);
                    templateObj.$.ti_widget_label_showless.removeAttribute('hidden');
                    templateObj.$.ti_widget_vtab_home.style.backgroundColor = "white";
                    templateObj.$.ti_widget_tabcontainer.selectedIndex = 1;
                });
            }
            if (templateObj.$.ti_widget_label_showless) {
                templateObj.$.ti_widget_label_showless.addEventListener("click", function (event) {
                    templateObj.$.ti_widget_label_productname.fontColor = "white";
                    templateObj.$.ti_widget_label_productdesc.fontColor = "white";
                    templateObj.$.ti_widget_label_learnmore.removeAttribute('hidden');
                    templateObj.$.ti_widget_label_showless.setAttribute('hidden', true);
                    templateObj.$.ti_widget_vtab_home.style.backgroundColor = "rgb(20, 140, 156)";
                    templateObj.$.ti_widget_tabcontainer.selectedIndex = 0;
                });
            }
        }, 1);

    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init.bind(this))
}